import os
import PIL
from PIL import Image
import torch
from torchvision.datasets.coco import CocoDetection
from bbsofa.metadata.filament_metadata import FilamentMetadata


def _has_only_empty_bbox(anno):
    """
    Checking if all the bounding boxes corresponding to an image have a height or width of less than 1 unit.

    If all the bounding boxes have height or width of less than 1 unit then
    considering the image have no bounding boxes.

    :param anno: list of annotations (bounding boxes) for an image.
    :return: True if the image has no bounding boxes, False otherwise.
    """
    return all(any(o <= 1 for o in obj["bbox"][2:]) for obj in anno)


def _has_valid_annotation(anno):
    """
    Checks if the image has valid bounding boxes.
    :param anno: list of annotations (bounding boxes) for an image.
    :return: True if the annotation list has valid bounding boxes.
    """
    # if it's empty, there is no annotation
    if len(anno) == 0:
        return False
    if _has_only_empty_bbox(anno):
        return False
    if "keypoints" not in anno[0]:
        return True
    return False


class _FilamentCutouts(CocoDetection):

    def __init__(self, ann_file: str, root: str, ids: list):
        """
        The class constructor initiates the class instance variables. It meanwhile identifies and excludes invalid
        annotations and images without annotations.

        :param ann_file: The JSON file with images and annotation data.
        :param root: the path to BBSO images.
        :param ids: image ids generated for particular timestamp
        """
        super().__init__(root, ann_file)
        self.ids: list = sorted(ids)
        filament_data = []
        ids = []
        for idx, img_id in enumerate(self.ids):
            ann_ids = self.coco.getAnnIds(imgIds=img_id, iscrowd=None)
            anno = self.coco.loadAnns(ann_ids)
            if _has_valid_annotation(anno):
                if len(anno) == 1:
                    filament_data.append([idx, anno[0]["category_id"]])
                else:
                    for a in anno:
                        filament_data.append([idx, a["category_id"]])
                ids.append(img_id)
        self.ids: list = ids
        self.filament_data: list = filament_data

    def _load_image(self, id: int) -> Image.Image:
        month, day, image_path = self.coco.loadImgs(id)[0]["file_name"].split(',')
        return Image.open(os.path.join(self.root, month, day, image_path))

    def get_filament_cutouts(self, idx: int) -> PIL:
        """
        This method uses the annotation dict to crop the filaments from full disk bbso images
        for a particular index.

        :param idx: index of the image list.
        :param filament_dir: filament directory provided.
        :return: cropped image (filament image).
        """
        img, anno = super().__getitem__(idx)
        img = img.convert("L")
        anno = [obj for obj in anno if obj['iscrowd'] == 0]
        boxes = [obj["bbox"] for obj in anno]
        boxes_xyxy: list = [0] * len(boxes)
        for i in range(len(boxes)):
            a_bbox = boxes[i]
            x0, y0 = a_bbox[0], a_bbox[1]
            x1, y1 = x0 + a_bbox[2], y0 + a_bbox[3]
            boxes_xyxy[i] = [x0, y0, x1, y1]

        boxes_xyxy = torch.as_tensor(boxes_xyxy).reshape(-1, 4)  # guard against no boxes

        img_crop = img.crop(
            (int(boxes_xyxy[0][0]), int(boxes_xyxy[0][1]), int(boxes_xyxy[0][2]), int(boxes_xyxy[0][3])))
        return img_crop



def main():
    bbso_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../..', 'petdata', '2015'))
    filament_info = FilamentMetadata('2015-08-05 17:36:15','2015-08-11 18:15:17')
    L, R, U = filament_info.get_chirality_distribution()
    print(L, R, U)
    # filament_cutout = _FilamentCutouts(_BBSO_JSON_FILE, bbso_path, filament_info.bbso_img_ids)
    # filament_cutout.get_filament_cutouts(2)
    # print("done")

if __name__ == "__main__":
    main()


import random
import cv2
import numpy as np
import torch
from torch.utils.data import DataLoader, Sampler
from typing import Iterator
from torch.utils.data._utils.fetch import _BaseDatasetFetcher
from torch.utils.data.dataloader import _BaseDataLoaderIter
from bbsofa.augment._filaments_augmentation import _FilamentAugmentation


class FilamentDataLoader(DataLoader):

    def __init__(self, dataset, batch_size: int, filament_ratio: tuple, n_batchs: int,
                 transforms: list, image_dim: int):
        """
        :param dataset: Filament Dataset object.
        :param batch_size: each batch size.
        :param filament_ratio: tuple of number of (L,R,U) chiralities for each batch.
        :param n_batchs: number of batches.
        :param transforms: transformations json file.
        :param image_dim: image dimensions ,i.e, square dimensions.
        :param _counter: dict of images and number of times the image is transformed.
        :param _removed: set of images that should not be used after a certain threshold.
        """
        if batch_size % sum(filament_ratio) != 0:
            raise Exception("batch size and filament ratio is not properly matched")
        self._dataset: list = dataset.data
        self.filament_cutout: list = dataset.filament_cutouts_data
        self.batch_size: int = batch_size
        self.n_l, self.n_r, self.n_u = filament_ratio
        if self.n_l < 0 or self.n_r < 0 or self.n_u < 0:
            raise Exception("ratio cannot be negative")
        self.filament_ratio: tuple = filament_ratio
        self._counter: dict = dict()
        self._removed: set = set()
        self.total_filaments: int = batch_size*n_batchs
        self.transforms: list = transforms
        self.image_dim: tuple  = (image_dim, image_dim)
        super().__init__(dataset.data, batch_size, shuffle=False, collate_fn=self.collate_fn,
                         sampler=CustomSampler(self.total_filaments), drop_last=False)

    def collate_fn(self, batch):
        """
        Augments the filaments based on the batch size, filament ratio.
        """
        filament_augmentation = _FilamentAugmentation(self.filament_cutout, self.batch_size,
                                                      self.filament_ratio, self.total_filaments,
                                                      self._counter, self._removed, self.transforms)
        c, rm = filament_augmentation.save_filaments()
        """
        Counter and removed list are updated for next iteration.
        """
        self._counter.update(c)
        self._removed = self._removed.union(rm)
        filament_list = filament_augmentation.augmented_data
        random.shuffle(filament_list)
        org_images = list()
        images = list()
        class_ids = list()
        """
        if image dimensions are -1 then the original image is stored in torch dataset
        else the image is resized based on the given image dimensions.
        """
        if self.image_dim != (-1, -1):
            for org_image,image, classes in filament_list:
                org_image = cv2.resize(np.array(org_image), self.image_dim)
                org_images.append(torch.from_numpy(org_image))
                image = cv2.resize(np.array(image), self.image_dim)
                images.append(torch.from_numpy(image))
                class_ids.append(torch.tensor([classes]))
            images = torch.stack(tuple(images))
            classes = torch.stack(tuple(class_ids))
        else:
            org_images, images, classes = zip(*filament_list)
        return org_images, images, classes

    def _get_iterator(self) -> '_BaseDataLoaderIter':
        """
        Iterates data loader for custom iteration class.
        """
        super(FilamentDataLoader, self)._get_iterator()
        if self.num_workers == 0:
            return _CustomSingleProcessDataLoaderIter(self)


class _CustomSingleProcessDataLoaderIter(_BaseDataLoaderIter):
    """
    This class is called when data loader is iterated.
    """
    def __init__(self, loader):
        super(_CustomSingleProcessDataLoaderIter, self).__init__(loader)
        assert self._timeout == 0
        assert self._num_workers == 0

        self._dataset_fetcher = _CustomDatasetKind.create_fetcher(
            0, self._dataset, self._auto_collation, self._collate_fn, self._drop_last)

    def _next_data(self):
        index = self._next_index()
        data = self._dataset_fetcher.fetch(index)
        return data


class CustomSampler(Sampler[int]):
    """
    Sampler defines number of iterations a data loader should perform.
    """
    def __init__(self, n_iterations) -> None:
        self.n_iterations = n_iterations

    def __iter__(self) -> Iterator[int]:
        return iter(range(1,self.n_iterations+1))

    def __len__(self) -> int:
        return self.n_iterations


class _CustomMapDatasetFetcher(_BaseDatasetFetcher):
    """
    This class is initialized everytime the dataset kind class is called for map dataset.
    """
    def __init__(self, dataset, auto_collation, collate_fn, drop_last):
        super(_CustomMapDatasetFetcher, self).__init__(dataset, auto_collation, collate_fn, drop_last)

    def fetch(self, idx):
        """
        Calls collate function every time iterator calls it.
        """
        if self.auto_collation:
            data = idx
        return self.collate_fn(data)


class _CustomDatasetKind(object):
    """
    Create_fetcher is called everytime the iter class initializes the datasetkind.
    """
    Map = 0
    @staticmethod
    def create_fetcher(kind, dataset, auto_collation, collate_fn, drop_last):
        if kind == _CustomDatasetKind.Map:
            return _CustomMapDatasetFetcher(dataset, auto_collation, collate_fn, drop_last)


__author__ = "Shreejaa Talla"

from datetime import datetime
from bbsofa.utils import file_utilities, timestamp_utilites


class FilamentMetadata:
    """
    This class provides  metadata about filaments based on the start and end timestamps.
    """
    def __init__(self, ann_file: str, start_time: str, end_time: str):
        """
        This constructor generates the list of images in between the start_time and end_time from BBSO data.
        Timestamp value in the format YYYY-MM-DD HH:MM:SS
        :param start_time: start_index time
        :param end_time: end time
        """
        self.start_time: datetime = datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S')
        self.end_time: datetime = datetime.strptime(end_time, '%Y-%m-%d %H:%M:%S')
        if self.start_time > self.end_time:
            raise Exception("start time provided can't be greater than end time")
        self.bbso_json: dict = file_utilities.read_json(ann_file)

    def parse_data(self):
        """
        based on the timestamp values, the annotation json file is parsed to find the closest timestamps
        available and initialize the images.
        """
        self.timestamp_dict: dict = timestamp_utilites.get_timestamp_dict(self.bbso_json)
        start_index: int = timestamp_utilites.find_closest(self.start_time, self.timestamp_dict)
        end_index: int = timestamp_utilites.find_closest(self.end_time, self.timestamp_dict)
        self.bbso_img_ids: list = list(self.timestamp_dict.keys())[start_index: end_index+1]

    def get_chirality_distribution(self) -> (int, int, int):
        """
        This method counts the number of left, right, and unidentified chirality f
        filaments in the given time interval, from BBSO data.

        :return: the number of left, right, and undefined filaments in the given time interval.
        """
        n_l = 0
        n_r = 0
        n_u = 0
        self.parse_data()
        annotations = self.bbso_json['annotations']
        for anno in annotations:
            if anno['image_id'] in self.bbso_img_ids:
                if anno['category_id'] == 0:
                    n_u += 1
                if anno['category_id'] == 1:
                    n_r += 1
                if anno['category_id'] == -1:
                    n_l += 1
        return n_l, n_r, n_u


def main():
    filamentInfo = FilamentMetadata('2015-08-05 17:36:15',
                                    '2015-08-11 18:15:17')
    L, R, U = filamentInfo.get_chirality_distribution()
    print(L, R, U)


if __name__ == "__main__":
    main()



import os
import PIL

from CONSTANTS import _LEFT_CHIRALITY, _RIGHT_CHIRALITY, _UNDEFINED_CHIRALITY
from bbsofa.cutouts._filament_cutouts import _FilamentCutouts
from bbsofa.metadata.filament_metadata import FilamentMetadata
from bbsofa.transform import _transformation
from bbsofa.transform._transformation import _Transformation


class _FilamentAugmentation:

    def __init__(self, filament_cutouts: _FilamentCutouts, batch_size: int, counts_per_batch: tuple,
                 total_filaments: int, _counter: dict, _removed: set, transforms: list):
        """
        Initializes the following parameters and throws exception if filament ratios are negative.
        :param filament_cutouts: Filament cutouts object
        :param batch_size: batch size for each iteration
        :param counts_per_batch: filament ratio in the format(L, R, U).
        :param total_filaments: total number of filaments, i.e, batch_size * number of batchs
        :param _counter: a dict of index and counts from previous iteration, else declares new one.
        :param _removed: a list of removed index from previous iteration or declares new one.
        """
        self.filament_cutouts: _FilamentCutouts = filament_cutouts
        self.ratio: tuple = counts_per_batch
        self.batch_size: int = batch_size
        self.dataset: list = self.filament_cutouts.filament_data
        self.n_l, self.n_r, self.n_u = counts_per_batch
        self.total_filaments: int = total_filaments
        if self.n_l < 0 or self.n_r < 0 or self.n_u < 0:
            raise Exception("Number of chiralities cannot be negative")
        self.augmented_data: list = list()
        self._counter: dict = _counter
        self._removed: set = _removed
        self.transforms: list = transforms

    def save_filaments(self):
        """
        For each chirality type based on batch and ratio,
         generate_filaments function is called.

        :return : counter dict and removed list.
        """
        n_times = self.batch_size // sum(self.ratio)
        if self.n_l != 0:
            self.__generate_filaments_each_type(_LEFT_CHIRALITY, n_times * self.n_l)
        if self.n_r != 0:
            self.__generate_filaments_each_type(_RIGHT_CHIRALITY, n_times * self.n_r)
        if self.n_u != 0:
            self.__generate_filaments_each_type(_UNDEFINED_CHIRALITY, n_times * self.n_u)
        return self._counter, self._removed

    def __get_filament_list(self, chirality_type: int) -> list:
        """
        This method get the list of annotation index for a chirality type.

        :param chirality_type: Left, right or undefined chiralities

        :return: list of index of annotations.
        """
        type_filaments = list()
        for x in self.dataset:
            if x[1] == chirality_type:
                type_filaments.append(x[0])
        return type_filaments

    def __cal_total_augments(self, chirality_type: int) -> float:
        """
        Calculate total number of filaments that should be augmented for each type.

        :param chirality_type: Left, right or unidentified

        :return: total augmentations
        """
        total = 0
        if chirality_type == _UNDEFINED_CHIRALITY:
            total = (self.n_u * self.total_filaments) / sum(self.ratio)
        elif chirality_type == _LEFT_CHIRALITY:
            total = (self.n_l * self.total_filaments) / sum(self.ratio)
        elif chirality_type == _RIGHT_CHIRALITY:
            total = (self.n_r * self.total_filaments) / sum(self.ratio)
        return total

    def __generate_filaments_each_type(self, chirality_type, n: int) -> None:
        """
        generates the extra filaments by performing transformations
        provided based on their types.

        :param chirality_type: Left, right or undefined chiralities.
        :param n: number of filament.

        :return: None.
        """
        count, filament_list, n_image_copies, remain = self.__initial_calculations(chirality_type)
        """
        Initializing counter for each chirality type.
        """
        if len(self._counter) == 0:
            self._counter.update({str(idx) + "_" + str(chirality_type): 0 for idx in filament_list})
        else:
            for idx in filament_list:
                if str(idx)+"_"+str(chirality_type) not in self._counter:
                    self._counter[str(idx) + "_" + str(chirality_type)] = 0
        if remain != 0 and n_image_copies == 0:
            """
            This part generated the remaining images, i.e., after all n_image_copies are 
            generated, sum of all n_image_copies is less than required copies then this part
            will be executed.
            """
            idx = [[k.split("_")[0], v] for k, v in self._counter.items()
                   if v < 1 and k.split("_")[1] == str(chirality_type)]
            for id, c in idx:
                count = self.__generate_image(c, chirality_type, count, id)
                if count >= n:
                    return
        idx = [[k.split("_")[0], v] for k, v in self._counter.items()
               if v < n_image_copies and k.split("_")[1] == str(chirality_type)]
        count, idx = self.__image_augments_original(chirality_type, count, idx, n, n_image_copies)
        self.__images_augments_remove_list(chirality_type, count, idx, n, n_image_copies)

    def __initial_calculations(self, chirality_type: int) -> int:
        """
        before augmentation, some calculations are made based on each type.
        :param chirality_type: left, right or unidentified.
        :return : total augmentations, n_image_copies, remain, count.
        """
        filament_list = self.__get_filament_list(chirality_type)
        length = len(filament_list)
        total = self.__cal_total_augments(chirality_type)
        n_image_copies = total // length
        remain = total % length
        count = 0
        return count, filament_list, n_image_copies, remain

    def __image_augments_original(self, chirality_type: int, count: int, idx: int, n: int, n_image_copies: int) -> int:
        """
        For each original filament cutout, n_image_copies are augmented and
        the counter is increase by 1 every time the same image index is called.
        For every loop only the indexes whose counter is not more than or equal to
        n_image_copies are used to generate images. This loop breaks when required
        left, right and unidentified augmentations are done based on batch_size.

        :param chirality_type: left, right or unidentified
        :param count: count_max == batch_size
        :param idx: index of annotation object.
        :param n: number of left/right/unidentified filaments generated for each batch.
        :param n_image_copies: number of copies/augmentations for each image.

        :return : count, idx
        """
        for i in range(count, n):
            if count >= n:
                break
            idx = [[k.split("_")[0], v] for k, v in self._counter.items()
                   if v < n_image_copies and k.split("_")[1] == str(chirality_type)]
            for id, c in idx:
                count = self.__generate_image(c, chirality_type, count, id)
                i += 1
                if count >= n:
                    break
        return count, idx

    def __images_augments_remove_list(self, chirality_type: int, count: int, idx: int, n: int, n_image_copies: int):
        """
        Remaining images are augmented from the removed list, i.e, after all n_image_copies are generated.
        At minimum 0 images are augmented from each image or at maximum 1 image.
        For every iteration counter picks up indexes which are augmented n_image_copies times.

        :param chirality_type: left, right or unidentified chirality.
        :param count: count_max == batch_size.
        :param idx: removed index of annotation object.
        :param n: number of left/right/unidentified filaments generated for each batch.
        :param n_image_copies: number of copies/augmentations for each image.
        """
        rm_idx = set(k.split("_")[0] for k, v in self._counter.items()
                     if v == n_image_copies and k.split("_")[1] == str(chirality_type))
        self._removed = self._removed.union(rm_idx)
        if len(idx) == 0 and count < n:
            for rm in rm_idx:
                c = self._counter.get(rm + "_" + str(chirality_type))
                count = self.__generate_image(c, chirality_type, count, rm)
                if count >= n:
                    break

    def __generate_image(self, c: int, chirality_type: int, count, id: int) -> int:
        """
        1. Updates the counter for each image augmentation.
        2. Gets the filament image based on the index.
        3. Transforms the image and append it to the main data.

        :param c: number of times an image is used for augmentation.
        :param chirality_type: left, right or unidentified chirality.
        :param count: number of images are being augmented.
        :param id: index of annotation object.

        :return : count
        """
        self._counter.update({str(id) + "_" + str(chirality_type): c + 1})
        image = self.filament_cutouts.get_filament_cutouts(int(id))
        self.augmented_data.append(self.__perform_transformations
                                   (chirality_type, image))
        count += 1
        return count

    def __perform_transformations(self, chirality_type: int, image: PIL) -> list:
        """
            This method performs the transformations on image based on the list of
            transforms functions provided.

            :param chirality_type: Left, right or undefined chirality.
            :param image: image file.

            :return: list of transformed image and type
            """
        transform = _Transformation(image, _transformation.get_transform(self.transforms))
        transformed_image = transform.transform_image()
        return [image, transformed_image, chirality_type]


def main():
    bbso_json = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '../..', 'petdata', 'bbso_json_data', '2015_chir_data.json'))
    transforms_json = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '../..', 'petdata', 'input_transformations', 'transforms.json'))
    bbso_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../..', 'petdata', '2015'))
    filament_info = FilamentMetadata(bbso_json,'2015-08-05 17:36:15', '2015-08-11 18:15:17')
    filament_info.parse_data()
    L, R, U = filament_info.get_chirality_distribution()
    print(L, R, U)
    filament_cutout = _FilamentCutouts(bbso_json, bbso_path, filament_info.bbso_img_ids)
    augment = _FilamentAugmentation(filament_cutout, 6, (1, 1, 1), 6,dict(),set(),transforms_json)
    augment.save_filaments()
    print(bbso_path)
    print(bbso_json)


if __name__ == "__main__":
    main()



import os

ROOT = os.path.dirname(__file__)
Requirements_path = os.path.join(ROOT,'requirements.txt')
_LEFT_CHIRALITY = -1
_RIGHT_CHIRALITY = 1
_UNDEFINED_CHIRALITY = 0
_BRIGHTNESS = "brightness"
_CONTRAST = "contrast"
_SATURATION = "saturation"
_ROTATION = "rotation"





import PIL
from PIL import Image


def get_image(image_path: str) -> PIL:
    try:
        image = Image.open(open(image_path, 'rb'))
    except FileNotFoundError as e:
        raise e
    return image



__author__ = "Shreejaa Talla"

import PIL
import random
from torchvision import transforms as torchvision_transforms


class _Transformation:
    """
    This class transforms the image by applying a method.
    """
    def __init__(self, image: PIL, transforms):
        """
        :param image: Image object
        :param transforms: the method applied to transform the image.
        """
        self.image: PIL = image
        self._transforms = transforms

    def transform_image(self) -> PIL:
        """
        Gets the image from given image path and performs transformation.
        :return: transformed image.
        """
        transformed_img = self._transforms(self.image)
        return transformed_img


def get_transform(transforms: list) -> torchvision_transforms.Compose:
    """
    This method forms a list of filter of randomly with random intensities using
    torch vision transformations.
    :return: the composed transforms using torchvision transforms.
    """
    random.shuffle(transforms)
    return torchvision_transforms.Compose(transforms)



import torch
import os
from torch.utils.data import Dataset
from bbsofa.cutouts._filament_cutouts import _FilamentCutouts
from bbsofa.metadata.filament_metadata import FilamentMetadata


class FilamentDataset(Dataset):

    def __init__(self, bbso_path: str, ann_file: str, start_time: str, end_time: str):
        """
        The constructor gets the image ids based on start and end time.
        based on the image ids, filaments annotation index and their respective class labels
        are initialized to dataset.
        :param bbso_path: path to bsso full disk images.
        :param ann_file: path to annotations file.
        :param start_time: start time in YYYY:MM:DD HH:MM:SS.
        :param end_time: end time in YYYY:MM:DD HH:MM:SS.
        """
        filament_metadata = FilamentMetadata(ann_file,start_time, end_time)
        filament_metadata.parse_data()
        self.bbso_img_ids: list = filament_metadata.bbso_img_ids
        self.filament_cutouts_data: _FilamentCutouts = _FilamentCutouts(ann_file, bbso_path, self.bbso_img_ids)
        self.data: list = self.filament_cutouts_data.filament_data

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        anno, class_name = self.data[idx]
        anno_tensor = torch.from_numpy(anno)
        class_id = torch.tensor([class_name])
        return anno_tensor, class_id


if __name__ == "__main__":
    bbso_json = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '../..', 'petdata', 'bbso_json_data', '2015_chir_data.json'))
    transforms_json = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '../..', 'petdata', 'input_transformations', 'transforms.json'))
    bbso_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../..', 'petdata', '2015'))
    dataset = FilamentDataset(bbso_path,bbso_json,"2015-08-05 17:36:15", "2015-08-11 18:15:17")
    print(type(dataset.data))
    # data_loader = FilamentDataLoader(dataset, 6, (1, 1, 1), 10)
    # print(len(data_loader))
    # for imgs, labels in data_loader:
    #     print("Batch of images has shape: ", imgs)
    #     print("Batch of labels has shape: ", labels)


__author__ = "Shreejaa Talla"
__source__ = "https://bitbucket.org/gsudmlab/filament_dataacquisition/src/master/data_acquisition/fileexplorer.py"
__modified__ = "yes"

import bisect
from sortedcontainers import SortedDict
from datetime import datetime


def get_timestamp_dict(bbso_json: dict) -> dict:
    """
    Gets a sorted dictionary with all timestamp values in bbso json file as keys and
    image ids as values.
    :param bbso_json: BBSO json file as input
    :return: the timestamp and image ids dictonary
    """
    images = bbso_json["images"]
    timestamp_dict = SortedDict({})
    for image in images:
        dt = datetime.strptime(image["date_captured"], '%Y-%m-%d %H:%M:%S')
        timestamp_dict[image['id']] = dt
    return timestamp_dict


def find_closest(q_time: datetime, timestamp_dict: dict) -> int:
    """
    finds the closest timestamp value for given q-time and return the index of timestamp
    dict.
    :param timestamp_dict: the dictionary of timestamps as keys and image ids as values.
    :param q_time: datetime for which the image id should be queried.
    :return: index of the closest timestamp
    Note: this code is based on the suggestions here:
            - https://stackoverflow.com/questions/8162379/python-locating-the-closest-timestamp
            - https://docs.python.org/2/library/datetime.html#strftime-and-strptime-behavior
    """
    q_index = bisect.bisect_left(timestamp_dict.values(), q_time)
    if q_index >= len(timestamp_dict):  # if q_time occurs after the last time stamp
        q_index = q_index - 1
    elif q_index == 0:
        pass
    else:
        before_q_time = timestamp_dict.items()[q_index - 1][1]
        after_q_time = timestamp_dict.items()[q_index][1]
        diff_from_before = q_time - before_q_time
        diff_from_after = after_q_time - q_time
        q_index = (q_index if diff_from_after < diff_from_before else q_index - 1)
    return q_index



import os
import setuptools
import CONSTANTS
from setuptools import find_packages

readme = "README.md"
requirement_path = CONSTANTS.Requirements_path
readme_path = os.path.join(os.path.dirname(__file__), readme)

with open(readme, "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open(requirement_path) as f:
    install_requires = f.read().splitlines()
# pkgs = ['bbsofa', 'bbsofa.augment', 'bbsofa.cutouts',
#         'bbsofa.loader', 'bbsofa.metadata', 'bbsofa.transform',
#         'bbsofa.utils']
pkgs = find_packages()
print(pkgs)

setuptools.setup(
    name="demo_fa",
    version="0.0.3",
    author="Shreejaa Talla",
    author_email="shreejaa.talla@gmail.com",
    description="Solar Filaments data augmentation demo package",
    url="https://bitbucket.org/gsudmlab/bbso_fa/src/master/",
    project_urls={
        "Source": "https://bitbucket.org/gsudmlab/bbso_fa/src/master/",
    },
    packages = pkgs,
    package_data={
        '.': ['requirements.txt']},
    install_requires=install_requires,
    py_modules=['CONSTANTS'],
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],

)




import os
import json


def create_dir(dir_name: str):
    """
    create a directory if doesnt exists.
    :param dir_name: directory name
    :return: None
    """
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)


def read_json(json_file: str) -> dict:
    """
    Reads json file from the given path
    :param json_file: json file path
    :return: json file
    """
    with open(json_file) as json_file:
        json_data = json.load(json_file)
    return json_data


def main():
    json_file = read_json(r'/petdata/input_transformations/transforms.json')
    print('brightness' in json_file.keys())


if __name__ == "__main__":
    main()